import java.util.*;
import java.io.*;
import java.net.*;
import java.text.SimpleDateFormat;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.w3c.dom.Element;


public class  SMTPBridge {
  
  public static ServerSocket s = null;
  public static Socket incoming = null;
  public static String configFile = "config.xml";
  public static Integer Port = 25;
  public static Integer LogLevel = 1; // 0 - 2
  public static Map<String,HashMap<String, String>> Triggers = new HashMap<String, HashMap<String,String>>(); 
  
  public static void main(String[] args) {
      LogWrite("SMTPBridge V0.5 by STELE",0);
      LogWrite("starting up...",0);
      if(!loadSettings()){
        LogWrite("ERROR reading settings.xml",0);
        return;
      }

      String m_from = "";
      String m_to   = "";
      
      
      while (true) {
        
        try {
            // establish server socket
            LogWrite("Open Server Socket Port : "+ Port, 2);
            s = new ServerSocket(Port);
            LogWrite("SMTPBridge ist listening on port : " + Port, 1);
            
            // wait for incoming connection
            incoming = s.accept();
            
            InputStream inps = incoming.getInputStream();
            OutputStream outs = incoming.getOutputStream();
            
            Scanner in = new Scanner(inps);
            PrintWriter out = new PrintWriter(outs, true);
            
            out.println("220 Service ready");
            
            boolean done = false;
            boolean mailcont = false;
            String Email = "";
            String cmd   = "";
            while (!done && in.hasNextLine()) {
              // read data from Socket
              String line = in.nextLine();
              // Create command
              LogWrite(line,3);
              if(!mailcont && line.trim().length() >= 4){
                cmd = line.substring(0,4).toUpperCase();                 
              }else if (mailcont && (line.equals("."))){
                cmd = ".";
                mailcont = false;
              }
              // Email Body
              if(mailcont){
                //Email = Email + "\r\n" + line;
              } else {
                LogWrite("New CMD: " + line, 2);
                switch(cmd){
                  case "HELO": out.println("250 OK"); 
                               break;
                  case "EHLO": out.println("250 OK"); 
                               break;
                  case "MAIL": 
                              m_from = GetMail(line);
                              out.println("250 OK, "+m_to+ " resolved");
                               break;
                  case "RCPT":
                              m_to = GetMail(line); 
                              if(CheckMail(m_from, m_to)){
                                out.println("250 OK, "+m_from+ " resolved");
                               } else {
                                 LogWrite("553 Sender and Recipient do not match.", 1);
                                 out.println("553 Sender and Recipient do not match.");
                                 done = true;
                               } 
                               break;
                  
                  case "DATA": out.println("354 start mail input");
                               mailcont = true;  
                               break;
                  case ".":    out.println("250 OK");
                               mailcont = false;  
                               break;
                  case "QUIT": out.println("221 closing channel");
                               // Save the Mail
                               //SaveMail(Email);
                               if(CheckMail(m_from, m_to)) TriggerMail(m_from, m_to);                               
                               done = true; 
                               break;
                  default:     out.println("500 Syntax error, command unrecognised");
                               break;    
                }
                cmd = "";
              } // Body

            }

            
        } catch (IOException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
        
        try {
          incoming.close();
          incoming = null;
        } catch (IOException e) {
          e.printStackTrace();
        }
        
        try {
          s.close();
          s = null;
        } catch (IOException e) {
          e.printStackTrace();
        }
      
      } /* WHILE */
    
    }  /* Void Main */
    
    static boolean SaveMail(String Mail){
       FileWriter writer;
       File file;
       file = new File("mail.dat");
       try {
             writer = new FileWriter(file ,true);
             writer.write(Mail);
             writer.flush();
             
             // Schließt den Stream
             writer.close();
             return true;
          } catch (IOException e) {
            e.printStackTrace();
            return false;
          }       
    }
    
    static boolean loadSettings(){
      LogWrite("Loading config File: " + configFile, 0);
      try {
      	File fXmlFile = new File(configFile);

        
      	DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
      	DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
      	Document doc = dBuilder.parse(fXmlFile);
       
      	doc.getDocumentElement().normalize();
       	// Root ausgeben
        LogLevel =  Integer.parseInt(doc.getElementsByTagName("loglevel").item(0).getTextContent());
        LogWrite("LogLevel read: "+LogLevel, 0 );
        
        LogWrite("Reading listenport...", 2);
        Port =  Integer.parseInt(doc.getElementsByTagName("listenport").item(0).getTextContent());
        LogWrite("SMTP Listenport:" + Port, 2);
        
        LogWrite("Reading <trigger> entries... ", 2);
        NodeList nList = doc.getElementsByTagName("trigger");

        LogWrite(nList.getLength() + " trigger entries found", 2);
       	for (int temp = 0; temp < nList.getLength(); temp++) {
       		Node nNode = nList.item(temp);
       		if (nNode.getNodeType() == Node.ELEMENT_NODE) {
       			Element eElement = (Element) nNode;
       			
            String Triggerid = eElement.getAttribute("id");
            if(Triggerid.trim() == ""){
              LogWrite("ERROR: Trigger ID not set in Trigger Entry "+temp,0);
            }

            LogWrite("Loading Trigger with ID " + Triggerid, 3);
            Triggers.put(Triggerid, new HashMap<String,String>());
            Triggers.get(Triggerid).put("from", eElement.getElementsByTagName("from").item(0).getTextContent());
            Triggers.get(Triggerid).put("to", eElement.getElementsByTagName("to").item(0).getTextContent());      
            Triggers.get(Triggerid).put("type", eElement.getElementsByTagName("type").item(0).getTextContent());
            Triggers.get(Triggerid).put("command", eElement.getElementsByTagName("command").item(0).getTextContent());
            Triggers.get(Triggerid).put("host", eElement.getElementsByTagName("host").item(0).getTextContent());
            Triggers.get(Triggerid).put("port", eElement.getElementsByTagName("port").item(0).getTextContent());
            LogWrite("Trigger read: " + Triggers.get(Triggerid), 3);      
       		}
      	}
        LogWrite("Triggers read successfully.",2);
        return true;
      } catch (Exception e) {
    	   LogWrite("ERROR Parsing Config XML: "+ e.getMessage(), 0);
         return false;
    } 
        
    }
    
    static String GetMail(String mail){
          mail = mail.toLowerCase().replace("mail from:", "").trim();    
          mail = mail.toLowerCase().replace("rcpt to:", "").trim();
          mail = mail.replace("<","");
          mail = mail.replace(">","");
          return mail;
    }    

    static boolean CheckMail(String m_from, String m_to){
      boolean found = false;
      LogWrite("MailCheck FROM: "+ m_from+" and Mail TO: " + m_to, 2);
      for(Map.Entry<String, HashMap<String,String>> Trigger : Triggers.entrySet()){
        if(Trigger.getValue().get("from").toLowerCase().equals(m_from.toLowerCase()) &&
           Trigger.getValue().get("to").toLowerCase().equals(m_to.toLowerCase())){
           found = true;
           LogWrite("Sender, Recipiend do match. OK ", 2);
           }        
       }
      if(!found)LogWrite("Sender, Recipiend do NOT match. Abort ", 2);
          
      return found;
    }
    
    static  void TriggerMail(String m_from, String m_to){
      for(Map.Entry<String, HashMap<String,String>> Trigger : Triggers.entrySet()){
        if(Trigger.getValue().get("from").toLowerCase().equals(m_from) &&
           Trigger.getValue().get("to").toLowerCase().equals(m_to)){
           // Now execute the trigger
            String type    = Trigger.getValue().get("type").toLowerCase();  
            String trigger = Trigger.getValue().get("command");
            trigger = trigger.replace("%from%", m_from).replace("%to%", m_to);
            
            if(type.equals("exec")){
              LogWrite("Trigger, type EXEC: " + trigger, 1);
              System.out.println("EXEC command detected: " + trigger);
              Runtime r = Runtime.getRuntime();
              Process p = null;
              try {
                  p = r.exec(trigger);
                  LogWrite("Trigger executed.\n", 1);
              } catch (Exception e) {
                  LogWrite("ERROR EXEC: "+ e.getMessage(), 0);
              }            
            }
            
            if(type.equals("sock")){
              int port = Integer.parseInt(Trigger.getValue().get("port"));
              String host = Trigger.getValue().get("host").trim();    
              try{
                LogWrite("Trigger, type SOCK: " + host + ":" + port + ", "+ trigger, 1);
                  // ip and port where is my device  
                  Socket socket = new Socket(host, port);
                  DataOutputStream dataOutputStream = new DataOutputStream(socket.getOutputStream());
      
                  // i need send this parameter to my device 
                  dataOutputStream.writeUTF(trigger);
                  dataOutputStream.flush();
                  dataOutputStream.close();
                  socket.close();
                  LogWrite("Sock Command sent.\n",1);
      
              } catch (UnknownHostException ex) {
                  LogWrite("ERROR SOCK: "+ ex.getMessage(), 0);
              } catch (IOException ex) {
                  LogWrite("ERROR SOCK: "+ ex.getMessage(), 0);
              }
            }
           }        
       }
    }
    
  static void LogWrite(String msg, Integer level){
          if(level <= LogLevel)
            System.out.println(SimpleDate() + ": " + msg);
         }
  static String SimpleDate() { 
      SimpleDateFormat formatter = new SimpleDateFormat( 
              "yyyy-MM-dd, HH:mm:ss "); 
      Date currentTime = new Date(); 
      return (formatter.format(currentTime));        // 2012.04.14 - 21:34:07     
  }    
    
    
} /* Class Server*/

